/**
 $id: BadMenu.c

 BadMenu - A simple menu for BadWM
 Copyright (C) Bernd Leitner <b at swag.at> 2004

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


**/

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "config.h"
#include "keymap.h"
#include "BadMenu.h"

#define BORDER_WIDTH 	 2
#define XOFF	 	20
#define YOFF		20
#define DRAW_YOFF	 5
#define FSIZE		12	


Display * dpy;            
Window root, win;
int screen;
int depth;

char color[10]="black";
char color_bg[10]="darkgray";

GC gc;
Font font;
char * window_name={"BadMenu v0.1"}; 
char selected_MenuItem[60];
char font_char[10];
int selected_id=0;
int x,y;
unsigned int VISIBLE = 1;
unsigned int fsize = FSIZE;



int main(int argc, char **argv){	
	int i, menu_h, menu_w;
	unsigned int xoff = XOFF;
        unsigned int yoff = YOFF;
	char cfg[1024];	
	XEvent ev;

	Menu * BadMenu = NULL;

	snprintf(cfg,1024,"%s/.BadMenu.conf", getenv("HOME"));
	snprintf(font_char,10,"12");
	
	/* Parse argv */
	for (i=1; i < argc; i++) {
	    if (!strcmp(argv[i],"-fg")){
	           snprintf(color,10,"%s",argv[i+1]);
	    }
	    else if (!strcmp(argv[i],"-bg")){
		   snprintf(color_bg,10,"%s",argv[i+1]);
	    }	
	    else if (!strcmp(argv[i],"-cfg")){
		   snprintf(cfg,1024,"%s",argv[i+1]);
	    }
	    else if (!strcmp(argv[i],"-xoff")){
	   	   if(atoi(argv[i+1]) < 0){
			fprintf(stderr,"Error::X offset can't be negative\n");
			exit(0);
		   }
		   xoff=atoi(argv[i+1]);
	    }	
	    else if (!strcmp(argv[i],"-yoff")){
	           if(atoi(argv[i+1]) < 0){
		        fprintf(stderr,"Error::Y offset can't be negative\n");
		        exit(0);
		   }
		   yoff=atoi(argv[i+1]);
     	    }
	    else if (!strcmp(argv[i],"-fs")){
                   if(atoi(argv[i+1]) < 0){
                        fprintf(stderr,"Error::Fontsize can't be negative\n");
                        exit(0);
                   }
                   fsize=atoi(argv[i+1]);
		   snprintf(font_char,strlen(argv[i+1])+1,argv[i+1]);
            }
	    else if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help")) {
	        printf("usage: BadMenu [-fg -bg -xoff -yoff -fs -cfg]\n");
		exit(0);
	    }
	}

	/* Config */
	BadMenu = get_MenuData(cfg);
	dpy = XOpenDisplay(NULL);
	if (dpy==NULL) {
	  	fprintf(stderr, "Error::Given display can't be opened");
	  	return;
	}
	
  	screen = DefaultScreen(dpy);
	depth = DefaultDepth(dpy,screen);
	root = RootWindow(dpy,screen);	
	
	if(fsize < 20) menu_h = (BadMenu->numItems * 15)+10; /* dirty hack */
	else	       menu_h = (BadMenu->numItems * 20)+10;

	menu_w = (BadMenu->maxItemLen * (fsize * 0.55));
	setup_BadMenu(0,BadMenu);
	open_Window(xoff,yoff,menu_w,menu_h,0,BadMenu);
	XSetInputFocus(dpy,win,FOCUS_REVERT,CurrentTime);
	
	/* Main Event Loop */
	while (1){ 
   		XNextEvent(dpy, &ev);
	        	switch (ev.type) {
	                	case KeyPress: BadMenu_Event(&ev.xkey,BadMenu); 
					break;
				case FocusOut: XLowerWindow(dpy,win); 	
					break;
				case EnterNotify: XRaiseWindow(dpy,win);
						  XSetInputFocus(dpy,win,FOCUS_REVERT,CurrentTime);
					break;
				case LeaveNotify: /* Do nothing */
					break;
		   		case Expose:   setup_BadMenu(1,BadMenu); 
					break;
	     		}
	    	
	}
	return 0;
}


void setup_BadMenu(char real,Menu * head){
	int yoff, draw_yoff;
	Menu * current; current=head;

	if(fsize<20) draw_yoff = DRAW_YOFF;
	else	     draw_yoff = 2*DRAW_YOFF;
	
	if(real) XClearWindow(dpy, win);
	yoff = 10;
	
	while(current!=NULL) {
		if(real) {
			if(current->Item->selected==1) {
				snprintf(selected_MenuItem,strlen(current->Item->title)+5,"[ %s ]",current->Item->title);
				XDrawString(dpy,win,gc,5,yoff+draw_yoff,selected_MenuItem,strlen(selected_MenuItem));
			} 
			else 	XDrawString(dpy,win,gc,5,yoff+draw_yoff,current->Item->title,strlen(current->Item->title));
				
		}
		if(fsize < 20) yoff += 15;
		else	       yoff += 20;
		current=current->next;
	}
}



/* Now do this by fork()ing twice so we don't have to worry about SIGCHLDs */
void spawn(const char cmd[]) {
        pid_t pid;

        if (dpy)
                putenv(dpy);
        if (!(pid = fork())) {
                setsid();
                switch (fork()) {
                        case 0: system(cmd);
                        default: _exit(0);
                }
        }
        if (pid > 0)
                wait(NULL);
}

unsigned long get_Color(char * color) {  
	Colormap cmap;
  	XColor c0, c1;
  	cmap = DefaultColormap(dpy, 0);
  	
	XAllocNamedColor(dpy, cmap, color, &c1, &c0);
  	return(c1.pixel);
}

void BadMenu_Event(XKeyEvent * e, Menu * head){
	Menu * current; current = head;
	KeySym key = XKeycodeToKeysym(dpy,e->keycode,0);
	
	switch(key) {
		case KEY_LEFT: /* NOT YET */ 
			break;
		case KEY_DOWN: select_MenuItem(head,DOWN); setup_BadMenu(1,head); 
			break;
		case KEY_RIGHT: /* NOT YET */ 
			break;
		case KEY_UP: select_MenuItem(head,UP); setup_BadMenu(1,head); 
			break;
		case RETURN: while(current != NULL) {
			     	if(current->Item->id == selected_id) {
					spawn(current->Item->cmd);
					return;
				}	
				current=current->next;
			     }
			break;
		case CTRL_L:  if(!VISIBLE) { XRaiseWindow(dpy,win); VISIBLE = 1; }
			      else 	   { XLowerWindow(dpy,win); VISIBLE = 0; }	
			break;			
	}
}

void select_MenuItem(Menu * head, MenuMove choice) {
	
	switch(choice) {
		case LEFT: /* NOT YET */ 
			break;
		case DOWN: if((selected_id+1) == head->numItems) selected_id=0; 
			   else selected_id++; 
			   update_selectedToken(head); 
			break;
		case RIGHT:  /* NOT YET */
			break;
		case UP:   if((selected_id-1) == -1) selected_id=(head->numItems-1); 
			   else selected_id--;
			   update_selectedToken(head); 
			break;
	}
}

void update_selectedToken(Menu * head) {
	Menu * current; current = head;
	
	while(current!=NULL) {
		if(current->Item->id == selected_id) current->Item->selected=1;
		else current->Item->selected=0;

		current=current->next;
	}
}

void open_Window(int x, int y, int width, int height, int flag, Menu * head){
        XSetWindowAttributes attributes;
        XSizeHints sizehints;
	unsigned long mask;
	char font_str[60];

	snprintf(font_str,strlen(font_char)+35,"-misc-fixed-*-*-*-*-%s-*-*-*-*-*-*-*",font_char);
        attributes.border_pixel = BlackPixel(dpy,screen);
	attributes.background_pixel = get_Color(color_bg);
        attributes.override_redirect = True;
	attributes.event_mask = KeyPressMask|KeyReleaseMask|ExposureMask|VisibilityChangeMask|
				FocusChangeMask|EnterWindowMask|LeaveWindowMask;
	
	mask = CWBackPixel|CWBorderPixel|CWOverrideRedirect;
	win = XCreateWindow(dpy,RootWindow(dpy,screen),x,y,width,height,
                            BORDER_WIDTH,depth,InputOutput,CopyFromParent,
                            mask,&attributes);
	
	XStoreName(dpy,win,window_name);
        gc = XCreateGC(dpy,win,0,NULL);
	XSetForeground(dpy,gc,get_Color(color));
	font = XLoadFont(dpy,font_str);
        XSetFont(dpy,gc,font);

	sizehints.flags = PPosition | PSize;
	sizehints.x = x;
 	sizehints.y = y;
	sizehints.width = width;
	sizehints.height = height;
	
	XSetNormalHints(dpy,win,&sizehints);
	XChangeWindowAttributes(dpy, win, CWEventMask, &attributes);
	
	setup_BadMenu(1,head); 
	XMapWindow(dpy,win);
	XFlush(dpy);

}


/* EOF */
